#include "Programm.h"

int main(void) {

	cli();		// Interrupts deaktivieren
	init();		// Programm-Initialisierung
	lcd_init();	// Initialisierung von Display
	sei();		// Interrupts aktivieren	

	// Display ansteuern
	lcd_string(1, 1, "********************");	// 1. Zeile
	lcd_string(2, 1, "*  Arduino-Tester  *");	// 2. Zeile
	lcd_string(3, 1, "* wird gestartet...*");	// 3. Zeile
	lcd_string(4, 1, "********************");	// 4. Zeile
	_delay_ms(3000);
	lcd_string(2, 1, "*  Entwickelt von  *");
	lcd_string(3, 1, "*  Matthias Rock   *");
	_delay_ms(3000);
	lcd_string(2, 1, "*  Arduino-Tester  *");
	lcd_string(3, 1, "* wird gestartet...*");

	while(!new_display_data && !status) {

		lcd_string(3, 17, ".  ");
		_delay_ms(800);
		lcd_string(3, 17, ".. ");
		_delay_ms(800);
		lcd_string(3, 17, "...");
		_delay_ms(800);

	}
	// Endlos-Schleife
	while(1) {

		// Wenn sich die Displaydaten gendert haben, dann Daten an Display senden
		if(new_display_data) {		
						
			lcd_string(1, 1, (const char*)display_data[0]);	// 1. Zeile
			lcd_string(2, 1, (const char*)display_data[1]);	// 2. Zeile
			lcd_string(3, 1, (const char*)display_data[2]);	// 3. Zeile
			lcd_string(4, 1, (const char*)display_data[3]);	// 4. Zeile
			new_display_data = FALSE;
			I2C_ACK;										// Wieder bereit zum Empfang von neuen Daten

		}
		// Wenn ein neues Steuerbyte empfangen wurde
		if(status) {

			switch(status) {
				
				// Relais 1 = ON
				case 1:		switch_relais(1, ON);
							break;
				// Relais 1 = OFF
				case 2:		switch_relais(1, OFF);
							break;
				// Relais 2 = ON
				case 3:		switch_relais(2, ON);
							break;
				// Relais 2 = OFF
				case 4:		switch_relais(2, OFF);
							break;
				// Relais 3 = ON
				case 5:		switch_relais(3, ON);
							break;
				// Relais 3 = OFF
				case 6:		switch_relais(3, OFF);
							break;
				// Relais 4 = ON
				case 7:		switch_relais(4, ON);
							break;
				// Relais 4 = OFF
				case 8:		switch_relais(4, OFF);
							break;
				// Relais 5 = ON
				case 9:		switch_relais(5, ON);
							break;
				// Relais 5 = OFF
				case 10:	switch_relais(5, OFF);
							break;
				// Relais 6 = ON
				case 11:	switch_relais(6, ON);
							break;
				// Relais 6 = OFF
				case 12:	switch_relais(6, OFF);
							break;
				// Relais 7 = ON
				case 13:	switch_relais(7, ON);
							break;
				// Relais 7 = OFF
				case 14:	switch_relais(7, OFF);
							break;
				// Relais 8 = ON
				case 15:	switch_relais(8, ON);
							break;
				// Relais 8 = OFF
				case 16:	switch_relais(8, OFF);
							break;
				// AREF messen
				case 17:	analog_value = read_ADU(7);	// Spannung an ADC7 wird gemessen
							break;
				// SIG messen
				case 18:	analog_value = read_ADU(6);	// Spannung an ADC6 wird gemessen
							break;
				// Strom messen
				case 19:	analog_value = read_ADU(2);	// Spannung an PC2 wird gemessen
							break;
				// Alles auf den Initialisierungszustand zurcksetzen
				case 21:	init();
							break;
				default:	break;		

			}
			status = 0;
			I2C_ACK;					// Wieder bereit zum Empfang von neuen Daten

		}

	}
	return 0;

}
//************************************************************************
// Programm initialisieren
//************************************************************************
void init() {
	
	for(uint8_t i = 1; i <= 8; i++) switch_relais(i, OFF);	// Alle Relais auf OFF schalten	
	
	// Alle Relais-Pins als Ausgang schalten
	DDRC |= (1<<1);		// Relais 1 (PC1)
	DDRC |= (1<<0);		// Relais 2 (PC0)
	DDRB |= (1<<2);		// Relais 3 (PB2)
	DDRB |= (1<<1);		// Relais 4 (PB1)
	DDRB |= (1<<0);		// Relais 5 (PB0)
	DDRD |= (1<<7);		// Relais 6 (PD7)
	DDRD |= (1<<6);		// Relais 7 (PD6)
	DDRC |= (1<<3);		// Relais 8 (PC3)
	
	// Pin fr Strom-Messung (PC2) als Eingang schalten
	DDRC &= ~(1<<2);

	// Interner Pullup-Widerstand fr PC2 deaktivieren
	PORTC &= ~(1<<2);

	// Variablen initialisieren
	display_data[0][20] = 0;	// 1. Zeile
	display_data[1][20] = 0;	// 2. Zeile
	display_data[2][20] = 0;	// 3. Zeile
	display_data[3][20] = 0;	// 4. Zeile
	display_line_index = -1;	// Sorgt dafr, dass die ersten empfangenen Daten nicht als Displaydaten interpretiert werden
	display_col_index = 0;
	analog_value = 0xF000;		// Dies ist ein ungltiger Wert
	analog_byte_index = 0;
	new_display_data = FALSE;
	status = 0;

	// ADU initialisieren
	ADMUX = 0;
	ADMUX |= (1<<REFS0);				// Referenzspannung = AVCC
	ADCSRA = (1<<ADEN);					// ADU aktivieren, Single Conversion
	ADCSRA |= (1<<ADPS1) | (1<<ADPS2);	// Teilungsfaktor = 64 => 125 kHz
	ADCSRA |= (1<<ADSC);				// Eine Dummy-Umwandlung durchfhren
	while(ADCSRA & (1<<ADSC));			// Solange warten, bis Umwandlung fertig ist
	(void)ADCL;							// ADCL lesen, damit nchste Umwandlung bernommen werden kann
	(void)ADCH;							// ADCH lesen, damit nchste Umwandlung bernommen werden kann

	// I2C initialisieren
	TWAR = SLAVE_ADDRESS << 1;			// 7Bit-Slave-Adresse; LSB = Enable "General Call" (disabled)
	TWCR = (1<<TWEN);					// Aktiviert die I2C-Schnittstelle
	TWCR |= (1<<TWIE);					// Aktiviert den Interrupt
	TWCR |= (1<<TWEA);					// Aktiviert den "Besttigungs-Puls"	

}
//************************************************************************
// Relais wird in den gewnschten Zustand geschaltet
//************************************************************************
void switch_relais(uint8_t number, enum RelState state) {

	// Wenn das Relais auf OFF geschaltet werden soll
	if(state == OFF) {
		
		// Relais in OFF-Zustand schalten
		switch(number) {
		
			// Relais 1
			case 1: PORTC &= ~(1<<1);	// PC1 = LOW
					break;
			// Relais 2
			case 2: PORTC &= ~(1<<0);	// PC0 = LOW
					break;
			// Relais 3
			case 3: PORTB &= ~(1<<2);	// PB2 = LOW
					break;
			// Relais 4
			case 4: PORTB &= ~(1<<1);	// PB1 = LOW
					break;
			// Relais 5
			case 5: PORTB &= ~(1<<0);	// PB0 = LOW
					break;
			// Relais 6
			case 6: PORTD &= ~(1<<7);	// PD7 = LOW
					break;
			// Relais 7
			case 7: PORTD &= ~(1<<6);	// PD6 = LOW
					break;
			// Relais 8
			case 8: PORTC &= ~(1<<3);	// PC3 = LOW
					break;
			default:break;

		}

	}
	//Wenn das Relais auf ON geschaltet werden soll
	else {
		
		// Relais in ON-Zustand schalten
		switch(number) {
		
			// Relais 1
			case 1: PORTC |= (1<<1);		// PC1 = HIGH
					break;
			// Relais 2
			case 2: switch_relais(3, OFF);	// Relais 3 = OFF
					switch_relais(4, OFF);	// Relais 4 = OFF
					PORTC |= (1<<0);		// PC0 = HIGH
					break;
			// Relais 3
			case 3: switch_relais(2, OFF);	// Relais 2 = OFF
					switch_relais(4, OFF);	// Relais 4 = OFF
					PORTB |= (1<<2);		// PB2 = HIGH
					break;
			// Relais 4
			case 4: switch_relais(2, OFF);	// Relais 2 = OFF
					switch_relais(3, OFF);	// Relais 3 = OFF
					PORTB |= (1<<1);		// PB1 = HIGH
					break;
			// Relais 5
			case 5: PORTB |= (1<<0);		// PB0 = HIGH
					break;
			// Relais 6
			case 6: PORTD |= (1<<7);		// PD7 = HIGH
					break;
			// Relais 7
			case 7: PORTD |= (1<<6);		// PD6 = HIGH
					break;
			// Relais 8
			case 8: PORTC |= (1<<3);		// PC3 = HIGH
					break;
			default:break;

		}

	}

}
//************************************************************************
// Analog-Digital-Umwandlung durchfhren
//************************************************************************
uint16_t read_ADU(uint8_t channel) {

	uint16_t result;	

	ADMUX = (ADMUX & 0xF0) | (channel & 0x0F);	// Whlt den entsprechenden Kanal/Pin aus
	ADCSRA |= (1<<ADSC);						// Fhrt eine Umwandlung durch
	while(ADCSRA & (1<<ADSC));					// Solange warten, bis Umwandlung fertig ist
	result = ADCL;
	result |= 0x03FF & (ADCH<<8);

	return result;

}
//************************************************************************
// Interrupt-Service-Routine von I2C
//************************************************************************
ISR(TWI_vect) {

	// Unterschiedliche Aktionen abhngig vom Status ausfhren
	switch(TWSR & 0xF8) {

		//-------------------------------------------------------------------
		// Adressiert zum Lesen von Daten + ACK gesendet
		//-------------------------------------------------------------------
		case 0x60:
			
			I2C_ACK;	// Empfange Datenbyte, danach ACK fr evtl. weiteres Datenbyte senden
			break;

		//-------------------------------------------------------------------
		// Zuvor adressiert zum Lesen + Datenbyte empfangen + ACK gesendet
		//-------------------------------------------------------------------
		case 0x80:

			// Wenn es sich bei dem empfangenen Byte um Displaydaten handelt
			if(display_line_index >= 0) {

				display_data[display_line_index][display_col_index] = TWDR;	// Byte in Variable schreiben
				display_col_index++;										// Spaltenindex erhhen
				
				// Wenn Zeile vollgeschrieben ist, dann nchste Zeile auswhlen
				if(display_col_index == 20) {

					display_line_index++;
					display_col_index = 0;

				}
				// Wenn noch ein oder mehrere Bytes empfangen werden sollen
				if(display_line_index <= 3) {
				
					I2C_ACK;	// Empfange Datenbyte, danach ACK fr evtl. weiteres Datenbyte senden

				}
				// Wenn alle Displaydaten empfangen wurden
				else {

					I2C_NACK;					// Noch nicht bereit fr dem Empfang weiterer Daten
					display_line_index = -1;	// Sorgt dafr, dass das nchste empfangene Byte nicht als Displaydaten interpretiert wird
					display_col_index = 0;
					new_display_data = TRUE;	// Daten im nchsten Schleifendurchlauf an Display senden

				}
				break;

			}
			// Wenn es sich bei dem empfangenen Byte nicht um Displaydaten handelt:

			// Wenn als nchstes 80 Bytes an Displaydaten empfangen werden
			if(TWDR == 20) {

				display_line_index = 0;
				display_col_index = 0;
				I2C_ACK;						// Empfange Byte, danach ACK fr evtl. weiteres Datebyte senden

			}
			// Wenn es sich um ein anderes Steuerbyte handelt
			else {

				status = TWDR;
				I2C_NACK;						// Noch nicht bereit fr dem Empfang weiterer Daten	
					
			}
			break;

		//-------------------------------------------------------------------
		// Adressiert zum Senden von Daten + ACK gesendet
		//-------------------------------------------------------------------
		case 0xA8:
		//-------------------------------------------------------------------
		// Zuvor adressiert zum Schreiben + Datenbyte gesendet + ACK empfangen
		//-------------------------------------------------------------------
		case 0xB8:

			// Wenn als nchstes das hherwertige Byte gesendet werden soll
			if(analog_byte_index == 0) {

				TWDR = (analog_value>>8) & 0xFF;							// Das hherwertige Byte in das Register schreiben				
				I2C_ACK;													// Byte senden, danach ACK erwarten fr weiteres Datenbyte
				// Wenn der Wert gltig ist
				if(analog_value <= 0x03FF) analog_byte_index++;				// Beim nchsten Mal das niederwertige Byte senden

			}

			// Wenn als nchstes das niederwertige Byte gesendet werden soll
			else {

				TWDR = analog_value & 0xFF;			// Das niederwertige Byte in das Register schreiben				
				I2C_NACK;							// Letztes Byte senden, danach NACK erwarten
				analog_value = 0xF000;				// Ungltiger Wert fr den Fall, dass nochmals ausgelesen wird ohne vorherige Messung
				analog_byte_index = 0;

			}
			break;
		
		//-------------------------------------------------------------------
		// Zuvor adressiert zum Lesen + Datenbyte empfangen + NACK gesendet
		//-------------------------------------------------------------------
		case 0x88:

		//-------------------------------------------------------------------
		// Zuvor adressiert zum Schreiben + Datenbyte gesendet + NACK empfangen
		//-------------------------------------------------------------------
		case 0xC0:		

		//-------------------------------------------------------------------
		// Zuvor adressiert zum Schreiben + Letztes Datenbyte gesendet (TWEA = 0) + ACK empfangen
		//-------------------------------------------------------------------
		case 0xC8:
		
		default:		
			I2C_ACK;
			break;

	}

}
