#ifndef PROGRAMM_H
#define PROGRAMM_H

#define F_CPU 8000000					// Taktfrequenz des C
#define SLAVE_ADDRESS 0x40				// Slave-Adresse fr I2C (0x03...0x4f, 0x61...0x77)

// Includes
#include <avr/io.h>	
#include <avr/interrupt.h>
#include <util/delay.h>
#include "lcd-routines.h"

// Makros fr I2C
#define I2C_ACK 	TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT)|(1<<TWEA); 	// Acknowledge
#define I2C_NACK 	TWCR = (1<<TWEN)|(1<<TWIE)|(1<<TWINT);				// Not-Achnowledge

// Neue Datentypen
enum RelState {OFF, ON};
enum Bool {FALSE, TRUE};

// Globale Variablen
volatile char display_data[4][21];		// Displaydaten
volatile int8_t display_line_index;		// Index von Displayzeile
volatile uint8_t display_col_index;		// Index von Displayspalte
volatile uint16_t analog_value;			// Speichert den ausgelesenen Wert vom ADU
volatile uint8_t analog_byte_index;		// Index von Byte des Analogwertes
volatile enum Bool new_display_data;	// Statusvariable: TRUE, wenn sich Displaydaten gendert haben
volatile uint8_t status;				// Statusvariable zum Zwischenspeichern von empfangenem Steuerbyte

// Programm initialisieren
void init();

// Relais wird in den gewnschten Zustand geschaltet
void switch_relais(uint8_t number, enum RelState state);

// Analog-Digital-Umwandlung durchfhren
uint16_t read_ADU(uint8_t channel);

#endif
